@@ -14,13 +14,16 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import collections
 import json
 import linecache
 import os
 import sys
 import warnings
-from collections import Counter, Iterable
+
+try:
+    from collections import Iterable, Mapping
+except ImportError:
+    from collections.abc import Iterable, Mapping
 from dataclasses import asdict, dataclass, field
 from io import open
 from multiprocessing import Pool
@@ -76,12 +79,12 @@ class ModelArgs:
     early_stopping_patience: int = 3
     encoding: str = None
     eval_batch_size: int = 8
-    evaluate_during_training: bool = False
+    evaluate_during_training: bool = True
     evaluate_during_training_silent: bool = True
     evaluate_during_training_steps: int = 2000
     evaluate_during_training_verbose: bool = False
     evaluate_each_epoch: bool = True
-    fp16: bool = True
+    fp16: bool = False
     gradient_accumulation_steps: int = 1
     learning_rate: float = 4e-5
     local_rank: int = -1
@@ -95,7 +98,7 @@ class ModelArgs:
     model_type: str = None
     multiprocessing_chunksize: int = -1
     n_gpu: int = 1
-    no_cache: bool = False
+    no_cache: bool = True
     no_save: bool = False
     not_saved_args: list = field(default_factory=list)
     num_train_epochs: int = 1
@@ -122,10 +125,10 @@ class ModelArgs:
     train_batch_size: int = 8
     train_custom_parameters_only: bool = False
     use_cached_eval_features: bool = False
-    use_early_stopping: bool = False
+    use_early_stopping: bool = True
     use_hf_datasets: bool = False
-    use_multiprocessing: bool = True
-    use_multiprocessing_for_evaluation: bool = True
+    use_multiprocessing: bool = False
+    use_multiprocessing_for_evaluation: bool = False
     wandb_kwargs: dict = field(default_factory=dict)
     wandb_project: str = None
     warmup_ratio: float = 0.06
@@ -305,7 +308,7 @@ def build_classification_dataset(
         logger.info(f" Features loaded from cache at {cached_features_file}")
         examples, labels = data
     else:
-        logger.info(" Converting to features started. Cache is not used.")
+        logger.debug(" Converting to features started. Cache is not used.")
 
         if len(data) == 3:
             # Sentence pair task
@@ -317,7 +320,13 @@ def build_classification_dataset(
         # If labels_map is defined, then labels need to be replaced with ints
         if args.labels_map and not args.regression:
             if multi_label:
-                labels = [[args.labels_map[l] for l in label] for label in labels]
+                if isinstance(labels[0], str):
+                    labels = [[int(1) if i in label.split(args.labels_sep) else int(0) for i in
+                               args.labels_list] for label in labels]
+                elif isinstance(labels[0], list) and isinstance(labels[0][0], int):
+                    labels = labels
+                else:
+                    labels = [[args.labels_map[l] for l in label] for label in labels]
             else:
                 labels = [args.labels_map[label] for label in labels]
 
@@ -395,7 +404,11 @@ class ClassificationDataset(Dataset):
 
 def map_labels_to_numeric(example, multi_label, args):
     if multi_label:
-        example["labels"] = [args.labels_map[label] for label in example["labels"]]
+        if isinstance(example["labels"][0], str):
+            example["labels"] = [int(1) if i in example["labels"].split(args.labels_sep) else int(0) for i in
+                                 args.labels_list]
+        else:
+            example["labels"] = [args.labels_map[label] for label in example["labels"]]
     else:
         example["labels"] = args.labels_map[example["labels"]]
 
@@ -847,17 +860,27 @@ class LazyClassificationDataset(Dataset):
 
             # If labels_map is defined, then labels need to be replaced with ints
             if self.args.labels_map:
-                label = self.args.labels_map[label]
+                if self.args.multi_label:
+                    if isinstance(label[0], str):
+                        label = [int(1) if i in label.split(self.args.labels_sep) else int(0) for i in
+                                 self.args.labels_map.keys()]
+                    else:
+                        label = [self.args.labels_map[l] for l in label]
+                else:
+                    label = self.args.labels_map[label]
             if self.args.regression:
                 label = torch.tensor(float(label), dtype=torch.float)
+            elif self.args.multi_label:
+                label = torch.tensor([int(l) for l in label], dtype=torch.long)
             else:
                 label = torch.tensor(int(label), dtype=torch.long)
 
             return (
-                self.tokenizer.encode_plus(
-                    text,
+                self.tokenizer(
+                    text=text,
+                    truncation=True,
+                    padding="max_length",
                     max_length=self.args.max_seq_length,
-                    pad_to_max_length=self.args.max_seq_length,
                     return_tensors="pt",
                 ),
                 label,
@@ -872,11 +895,11 @@ class LazyClassificationDataset(Dataset):
                 label = torch.tensor(int(label), dtype=torch.long)
 
             return (
-                self.tokenizer.encode_plus(
+                self.tokenizer(
                     text_a,
                     text_pair=text_b,
+                    padding="max_length",
                     max_length=self.args.max_seq_length,
-                    pad_to_max_length=self.args.max_seq_length,
                     return_tensors="pt",
                 ),
                 label,
@@ -888,11 +911,11 @@ class LazyClassificationDataset(Dataset):
 
 def flatten_results(results, parent_key="", sep="/"):
     out = []
-    if isinstance(results, collections.Mapping):
+    if isinstance(results, Mapping):
         for key, value in results.items():
             pkey = parent_key + sep + str(key) if parent_key else str(key)
             out.extend(flatten_results(value, parent_key=pkey).items())
-    elif isinstance(results, collections.Iterable):
+    elif isinstance(results, Iterable):
         for key, value in enumerate(results):
             pkey = parent_key + sep + str(key) if parent_key else str(key)
             out.extend(flatten_results(value, parent_key=pkey).items())
@@ -946,10 +969,12 @@ class FocalLoss(nn.Module):
 
     def __init__(
             self,
-            alpha: Optional[Union[float, Iterable]] = None,
+            alpha: Optional[Union[float, Iterable]] = 0.5,
             gamma: Real = 2.0,
             reduction: str = "mean",
             ignore_index: int = -100,
+            epsilon=1.e-9,
+            activation_type='softmax',
     ) -> None:
         super(FocalLoss, self).__init__()
         if (
@@ -971,22 +996,20 @@ class FocalLoss(nn.Module):
         self.gamma: Real = gamma
         self.reduction: str = reduction
         self.ignore_index: int = ignore_index
+        self.epsilon = epsilon
+        self.activation_type: str = activation_type
 
     def forward(self, input: torch.Tensor, target: torch.Tensor) -> torch.Tensor:
+        """
+        Forward pass of the criterion.
+        @param input: model's output, shape [N, C] meaning N: batch_size and C: num_classes
+        @param target: ground truth, shape [N], N: batch_size
+        @return: shape of [N]
+        """
         if not torch.is_tensor(input):
             raise TypeError(
                 "Input type is not a torch.Tensor. Got {}".format(type(input))
             )
-        if input.shape[0] != target.shape[0]:
-            raise ValueError(
-                f"First dimension of inputs and targets should be same shape. "
-                f"Got: {input.shape} and {target.shape}"
-            )
-        if len(input.shape) != 2 or len(target.shape) != 1:
-            raise ValueError(
-                f"input tensors should be of shape (N, C) and (N,). "
-                f"Got: {input.shape} and {target.shape}"
-            )
         if input.device != target.device:
             raise ValueError(
                 "input and target must be in the same device. Got: {}".format(
@@ -996,28 +1019,43 @@ class FocalLoss(nn.Module):
 
         # filter labels
         target = target.type(torch.long)
-        input_mask = target != self.ignore_index
-        target = target[input_mask]
-        input = input[input_mask]
-        # compute softmax over the classes axis
-        pt = F.softmax(input, dim=1)
-        logpt = F.log_softmax(input, dim=1)
 
-        # compute focal loss
-        pt = pt.gather(1, target.unsqueeze(-1)).squeeze()
-        logpt = logpt.gather(1, target.unsqueeze(-1)).squeeze()
-        focal_loss = -1 * (1 - pt) ** self.gamma * logpt
+        if self.activation_type == 'sigmoid':
+            multi_hot_key = target
+            logits = torch.sigmoid(input)
+            zero_hot_key = 1 - multi_hot_key
+            focal_loss = -self.alpha * multi_hot_key * \
+                         torch.pow((1 - logits), self.gamma) * \
+                         (logits + self.epsilon).log()
+            focal_loss += -(1 - self.alpha) * zero_hot_key * \
+                          torch.pow(logits, self.gamma) * \
+                          (1 - logits + self.epsilon).log()
+            weights = torch.ones_like(
+                focal_loss, dtype=focal_loss.dtype, device=focal_loss.device
+            )
+        else:
+            input_mask = target != self.ignore_index
+            target = target[input_mask]
+            input = input[input_mask]
+            # compute softmax over the classes axis
+            pt = F.softmax(input, dim=1)
+            logpt = F.log_softmax(input, dim=1)
 
-        weights = torch.ones_like(
-            focal_loss, dtype=focal_loss.dtype, device=focal_loss.device
-        )
-        if self.alpha is not None:
-            if isinstance(self.alpha, float):
-                alpha = torch.tensor(self.alpha, device=input.device)
-                weights = torch.where(target > 0, 1 - alpha, alpha)
-            elif torch.is_tensor(self.alpha):
-                alpha = self.alpha.to(input.device)
-                weights = alpha.gather(0, target)
+            # compute focal loss
+            pt = pt.gather(1, target.unsqueeze(-1)).squeeze()
+            logpt = logpt.gather(1, target.unsqueeze(-1)).squeeze()
+            focal_loss = -1 * (1 - pt) ** self.gamma * logpt
+
+            weights = torch.ones_like(
+                focal_loss, dtype=focal_loss.dtype, device=focal_loss.device
+            )
+            if self.alpha is not None:
+                if isinstance(self.alpha, float):
+                    alpha = torch.tensor(self.alpha, device=input.device)
+                    weights = torch.where(target > 0, 1 - alpha, alpha)
+                elif torch.is_tensor(self.alpha):
+                    alpha = self.alpha.to(input.device)
+                    weights = alpha.gather(0, target)
 
         tmp_loss = focal_loss * weights
         if self.reduction == "none":
@@ -1037,17 +1075,23 @@ class FocalLoss(nn.Module):
         return loss
 
 
-def init_loss(weight, device, args):
+def init_loss(weight, device, args, multi_label=False):
     if weight and args.loss_type:
         warnings.warn(
-            f"weight and args.loss_type parametters are set at the same time"
+            f"weight and args.loss_type parameters are set at the same time"
             f"will use weighted cross entropy loss. To use {args.loss_type} set weight to None"
         )
     if weight:
-        loss_fct = nn.CrossEntropyLoss(weight=torch.Tensor(weight).to(device))
+        if multi_label:
+            loss_fct = nn.BCEWithLogitsLoss(weight=torch.Tensor(weight).to(device))
+        else:
+            loss_fct = nn.CrossEntropyLoss(weight=torch.Tensor(weight).to(device))
     elif args.loss_type:
         if args.loss_type == "focal":
-            loss_fct = FocalLoss(**args.loss_args)
+            if multi_label:
+                loss_fct = FocalLoss(activation_type='sigmoid', **args.loss_args)
+            else:
+                loss_fct = FocalLoss(activation_type='softmax', **args.loss_args)
         else:
             raise NotImplementedError(f"unknown {args.loss_type} loss function")
     else:
