@@ -306,7 +306,7 @@ class BertClassificationModel:
                     assert num_labels == len(self.args.labels_list)
                 if self.args.labels_map:
                     try:
-                        assert list(self.args.labels_map.keys()) == self.args.labels_list
+                        assert sorted(list(self.args.labels_map.keys())) == self.args.labels_list
                     except AssertionError:
                         assert [
                                    int(key) for key in list(self.args.labels_map.keys())
@@ -359,10 +359,10 @@ class BertClassificationModel:
                 )
         else:
             self.device = "cpu"
-        logger.info(f"Device: {self.device}")
+        logger.debug(f"Device: {self.device}")
 
         self.loss_fct = init_loss(
-            weight=self.weight, device=self.device, args=self.args
+            weight=self.weight, device=self.device, args=self.args, multi_label=multi_label
         )
 
         if self.args.onnx:
@@ -451,9 +451,8 @@ class BertClassificationModel:
                 "wandb_project specified but wandb is not available. Wandb disabled."
             )
             self.args.wandb_project = None
-        if multi_label:
-            self.weight = None  # Not implemented for multilabel
         self.multi_label = multi_label
+        self.args.multi_label = multi_label
 
     def train_model(
             self,
@@ -490,10 +489,12 @@ class BertClassificationModel:
             show_running_loss = False
 
         if self.args.evaluate_during_training and eval_df is None:
-            raise ValueError(
+            logger.warning(
                 "evaluate_during_training is enabled but eval_df is not specified."
                 " Pass eval_df to model.train_model() if using evaluate_during_training."
+                " now disabling evaluate_during_training."
             )
+            self.args.evaluate_during_training = False
 
         if not output_dir:
             output_dir = self.args.output_dir
@@ -942,8 +943,7 @@ class BertClassificationModel:
                         if test_df is not None:
                             test_results, _, _ = self.eval_model(
                                 test_df,
-                                verbose=verbose
-                                        and args.evaluate_during_training_verbose,
+                                verbose=verbose and args.evaluate_during_training_verbose,
                                 silent=args.evaluate_during_training_silent,
                                 wandb_log=False,
                                 **kwargs,
@@ -1612,13 +1612,25 @@ class BertClassificationModel:
                     ]
                     for example in preds
                 ]
+                pred_labels = [
+                    [
+                        self._threshold(pred, threshold_values[i])
+                        for i, pred in enumerate(example)
+                    ]
+                    for example in preds
+                ]
             else:
                 mismatched = labels != [
                     [self._threshold(pred, threshold_values) for pred in example]
                     for example in preds
                 ]
+                pred_labels = [
+                    [self._threshold(pred, threshold_values) for pred in example]
+                    for example in preds
+                ]
         else:
             mismatched = labels != preds
+            pred_labels = preds
 
         if eval_examples:
             if not isinstance(eval_examples[0], InputExample):
@@ -1654,13 +1666,22 @@ class BertClassificationModel:
 
         if self.multi_label:
             label_ranking_score = label_ranking_average_precision_score(labels, preds)
-            return {**{"LRAP": label_ranking_score}, **extra_metrics}, wrong
+            acc = accuracy_score(labels, np.array(pred_labels))
+            precision, recall, f_score, true_sum = precision_recall_fscore_support(
+                labels, np.array(pred_labels), average='weighted')
+            return {**{
+                "LRAP": label_ranking_score, 
+                "acc": acc, 
+                "precision": precision,
+                "recall": recall,
+                "f1": f_score,
+                }, **extra_metrics}, wrong
         elif self.args.regression:
             return {**extra_metrics}, wrong
 
         mcc = matthews_corrcoef(labels, preds)
         acc = accuracy_score(labels, preds)
-        classification_report_str = classification_report(labels, preds)
+        classification_report_str = classification_report(labels, preds, digits=4, output_dict=True)
 
         if self.model.num_labels == 2:
             tn, fp, fn, tp = confusion_matrix(labels, preds, labels=[0, 1]).ravel()
@@ -1693,7 +1714,7 @@ class BertClassificationModel:
                             "precision": precision,
                             "recall": recall,
                             "f1": f_score,
-                            # "classification_report": classification_report_str,
+                            "classification_report": classification_report_str,
                         },
                         **extra_metrics,
                     },
@@ -1708,7 +1729,7 @@ class BertClassificationModel:
                 "precision": precision,
                 "recall": recall,
                 "f1": f_score,
-                # "classification_report": classification_report_str,
+                "classification_report": classification_report_str,
             }, **extra_metrics}, wrong
 
     def predict(self, to_predict):
@@ -1983,7 +2004,13 @@ class BertClassificationModel:
             inverse_labels_map = {
                 value: key for key, value in self.args.labels_map.items()
             }
-            preds = [inverse_labels_map[pred] for pred in preds]
+            if self.multi_label:
+                preds = [
+                    [inverse_labels_map[i] for i, label in enumerate(example) if label]
+                    for example in preds
+                ]
+            else:
+                preds = [inverse_labels_map[pred] for pred in preds]
 
         if self.config.output_hidden_states:
             return preds, model_outputs, all_embedding_outputs, all_layer_hidden_states
@@ -2035,8 +2062,10 @@ class BertClassificationModel:
         if loss_fct:
             logits = outputs[1]
             labels = inputs["labels"]
-
-            loss = loss_fct(logits.view(-1, num_labels), labels.view(-1))
+            if self.multi_label:
+                loss = loss_fct(logits.view(-1, num_labels), labels.view(-1, num_labels))
+            else:
+                loss = loss_fct(logits.view(-1, num_labels), labels.view(-1))
         return (loss, *outputs[1:])
 
     def _threshold(self, x, threshold):
