@@ -24,22 +24,6 @@ use_cuda = torch.cuda.is_available()
 os.environ["TOKENIZERS_PARALLELISM"] = "false"
 
 
-def build_dataset(data_df, label_vocab_path):
-    y = data_df['labels']
-    if os.path.exists(label_vocab_path):
-        label_id_map = json.load(open(label_vocab_path, 'r', encoding='utf-8'))
-    else:
-        id_label_map = {id: v for id, v in enumerate(set(y.tolist()))}
-        label_id_map = {v: k for k, v in id_label_map.items()}
-        json.dump(label_id_map, open(label_vocab_path, 'w', encoding='utf-8'), ensure_ascii=False, indent=4)
-    logger.debug(f"label vocab size: {len(label_id_map)}, label_vocab_path: {label_vocab_path}")
-
-    df = data_df.copy()
-    df.loc[:, 'labels'] = df.loc[:, 'labels'].map(lambda x: label_id_map.get(x))
-    data_df = df
-    return data_df, label_id_map
-
-
 class BertClassifier(ClassifierABC):
     def __init__(
             self,
@@ -51,6 +35,7 @@ class BertClassifier(ClassifierABC):
             batch_size=64,
             max_seq_length=128,
             multi_label=False,
+            labels_sep=',',
             args=None,
     ):
         """
@@ -63,6 +48,7 @@ class BertClassifier(ClassifierABC):
         @param batch_size:
         @param max_seq_length:
         @param multi_label:
+        @param labels_sep:
         @param args:
         """
         default_args = {
@@ -71,6 +57,7 @@ class BertClassifier(ClassifierABC):
             "num_train_epochs": num_epochs,
             "train_batch_size": batch_size,
             "best_model_dir": os.path.join(model_dir, 'best_model'),
+            "labels_sep": labels_sep,
         }
         train_args = BertClassificationArgs()
         if args and isinstance(args, dict):
@@ -94,6 +81,8 @@ class BertClassifier(ClassifierABC):
         self.train_args = train_args
         self.use_cuda = use_cuda
         self.multi_label = multi_label
+        self.labels_sep = labels_sep
+        self.label_vocab_path = os.path.join(self.model_dir, 'label_vocab.json')
         self.is_trained = False
 
     def __str__(self):
@@ -102,44 +91,63 @@ class BertClassifier(ClassifierABC):
     def train(
             self,
             data_list_or_path,
+            dev_data_list_or_path=None,
             header=None,
             names=('labels', 'text'),
             delimiter='\t',
-            test_size=0.1
+            test_size=0.1,
     ):
         """
         Train model with data_list_or_path and save model to model_dir
         @param data_list_or_path:
+        @param dev_data_list_or_path:
         @param header:
         @param names:
         @param delimiter:
         @param test_size:
         @return:
         """
-        logger.debug('train model...')
+        logger.debug('train model ...')
         SEED = 1
         set_seed(SEED)
         # load data
-        X, y, data_df = load_data(data_list_or_path, header=header, names=names, delimiter=delimiter)
+        X, y, data_df = load_data(data_list_or_path, header=header, names=names, delimiter=delimiter,
+                                  labels_sep=self.labels_sep, is_train=True)
         if self.model_dir:
             os.makedirs(self.model_dir, exist_ok=True)
-        label_vocab_path = os.path.join(self.model_dir, 'label_vocab.json')
-        if not self.multi_label:
-            data_df, self.label_id_map = build_dataset(data_df, label_vocab_path)
-        if test_size > 0:
-            train_data, dev_data = train_test_split(data_df, test_size=test_size, random_state=SEED)
-        else:
+        labels_map = self.build_labels_map(y, self.label_vocab_path, self.multi_label, self.labels_sep)
+        labels_list = sorted(list(labels_map.keys()))
+        if dev_data_list_or_path is not None:
+            dev_X, dev_y, dev_df = load_data(dev_data_list_or_path, header=header, names=names,
+                                             delimiter=delimiter, labels_sep=self.labels_sep,
+                                             is_train=False)
             train_data = data_df
-            dev_data = None
+            dev_data = dev_df
+        else:
+            if test_size > 0:
+                train_data, dev_data = train_test_split(data_df, test_size=test_size, random_state=SEED)
+            else:
+                train_data = data_df
+                dev_data = None
         logger.debug(f"train_data size: {len(train_data)}")
         logger.debug(f'train_data sample:\n{train_data[:3]}')
-        # train model
         if dev_data is not None and dev_data.size:
             logger.debug(f"dev_data size: {len(dev_data)}")
             logger.debug(f'dev_data sample:\n{dev_data[:3]}')
-            self.model.train_model(train_data, eval_df=dev_data)
+        # train model
+        if self.train_args.lazy_loading:
+            train_data = data_list_or_path
+            dev_data = dev_data_list_or_path
+        if dev_data is not None:
+            self.model.train_model(
+                train_data, eval_df=dev_data,
+                args={'labels_map': labels_map, 'labels_list': labels_list}
+            )
         else:
-            self.model.train_model(train_data)
+            self.model.train_model(
+                train_data,
+                args={'labels_map': labels_map, 'labels_list': labels_list}
+            )
         self.is_trained = True
         logger.debug('train model done')
 
@@ -147,7 +155,7 @@ class BertClassifier(ClassifierABC):
         """
         Predict labels and label probability for sentences.
         @param sentences: list, input text list, eg: [text1, text2, ...]
-        @return: predict_label, predict_prob
+        @return: predict_labels, predict_probs
         """
         if not self.is_trained:
             raise ValueError('model not trained.')
@@ -157,21 +165,23 @@ class BertClassifier(ClassifierABC):
             return predictions, raw_outputs
         else:
             # predict probability
-            id_label_map = {v: k for k, v in self.label_id_map.items()}
-            predict_labels = [id_label_map.get(i) for i in predictions]
-            predict_probs = [1 - np.exp(-raw_output[prediction]) for raw_output, prediction in
+            predict_probs = [1 - np.exp(-np.max(raw_output)) for raw_output, prediction in
                              zip(raw_outputs, predictions)]
-            return predict_labels, predict_probs
+            return predictions, predict_probs
 
     def evaluate_model(self, data_list_or_path, header=None,
-                       names=('labels', 'text'), delimiter='\t'):
-        X_test, y_test, data_df = load_data(data_list_or_path, header=header, names=names, delimiter=delimiter)
-        self.load_model()
-        if not self.multi_label:
-            data_df, _ = build_dataset(data_df, self.label_vocab_path)
+                       names=('labels', 'text'), delimiter='\t', **kwargs):
+        if self.train_args.lazy_loading:
+            eval_df = data_list_or_path
+        else:
+            X_test, y_test, eval_df = load_data(data_list_or_path, header=header, names=names, delimiter=delimiter,
+                                            labels_sep=self.labels_sep)
+        if not self.is_trained:
+            self.load_model()
         result, model_outputs, wrong_predictions = self.model.eval_model(
-            data_df,
+            eval_df,
             output_dir=self.model_dir,
+            **kwargs,
         )
         return result
 
@@ -182,11 +192,11 @@ class BertClassifier(ClassifierABC):
         """
         model_path = os.path.join(self.model_dir, 'pytorch_model.bin')
         if os.path.exists(model_path):
-            if not self.multi_label:
-                self.label_vocab_path = os.path.join(self.model_dir, 'label_vocab.json')
-                self.label_id_map = load_vocab(self.label_vocab_path)
-                num_classes = len(self.label_id_map)
-                assert num_classes == self.num_classes, f'num_classes not match, {num_classes} != {self.num_classes}'
+            labels_map = json.load(open(self.label_vocab_path, 'r', encoding='utf-8'))
+            labels_list = sorted(list(labels_map.keys()))
+            num_classes = len(labels_map)
+            assert num_classes == self.num_classes, f'num_classes not match, {num_classes} != {self.num_classes}'
+            self.train_args.update_from_dict({'labels_map': labels_map, 'labels_list': labels_list})
             self.model = BertClassificationModel(
                 model_type=self.model_type,
                 model_name=self.model_dir,
@@ -201,6 +211,34 @@ class BertClassifier(ClassifierABC):
             self.is_trained = False
         return self.is_trained
 
+    @staticmethod
+    def build_labels_map(y, label_vocab_path, multi_label=False, labels_sep=','):
+        """
+        Build labels map
+        @param y:
+        @param label_vocab_path:
+        @param multi_label:
+        @param labels_sep:
+        @return:
+        """
+        if multi_label:
+            labels = set()
+            for label in y.tolist():
+                if isinstance(label, str):
+                    labels.update(label.split(labels_sep))
+                elif isinstance(label, list):
+                    labels.update(range(len(label)))
+                else:
+                    labels.add(label)
+        else:
+            labels = set(y.tolist())
+        labels = sorted(list(labels))
+        id_label_map = {id: v for id, v in enumerate(labels)}
+        label_id_map = {v: k for k, v in id_label_map.items()}
+        json.dump(label_id_map, open(label_vocab_path, 'w', encoding='utf-8'), ensure_ascii=False, indent=4)
+        logger.debug(f"label vocab size: {len(label_id_map)}, label_vocab_path: {label_vocab_path}")
+        return label_id_map
+
 
 if __name__ == '__main__':
     parser = argparse.ArgumentParser(description='Bert Text Classification')
