@@ -5,6 +5,7 @@
 """
 
 import sys
+from abc import ABC
 
 import torch
 from torch import nn
@@ -118,6 +119,60 @@ class BertForMultiLabelSequenceClassification(BertPreTrainedModel):
         return outputs  # (loss), logits, (hidden_states), (attentions)
 
 
+class BertForHierarchicalMultiLabelSequenceClassification(BertPreTrainedModel):
+    def __init__(self, config, pos_weight=None):
+        super(BertForHierarchicalMultiLabelSequenceClassification, self).__init__(config)
+        config.update({"mlp_size": 1024})
+        self.num_labels = config.num_labels
+        self.bert = BertModel(config)
+        self.dropout = nn.Dropout(config.hidden_dropout_prob)
+        self.mlp = nn.Sequential(
+            nn.Linear(config.hidden_size + config.num_labels, config.mlp_size),
+            nn.ReLU(),
+            nn.Linear(config.mlp_size, config.mlp_size),
+            nn.ReLU(),
+        )
+        self.classifier = nn.Linear(config.mlp_size, config.num_labels)
+        self.pos_weight = pos_weight
+
+        self.init_weights()
+
+    def forward(
+            self,
+            input_ids,
+            attention_mask=None,
+            token_type_ids=None,
+            position_ids=None,
+            head_mask=None,
+            labels=None,
+            parent_labels=None,
+    ):
+        outputs = self.bert(
+            input_ids,
+            attention_mask=attention_mask,
+            token_type_ids=token_type_ids,
+            position_ids=position_ids,
+            head_mask=head_mask,
+        )
+        pooled_output = outputs[1]
+        pooled_output = self.dropout(pooled_output)
+        concat_output = torch.cat((pooled_output, parent_labels), dim=1)
+        mlp_output = self.mlp(concat_output)
+        logits = self.classifier(mlp_output)
+
+        outputs = (logits,) + outputs[2:]
+
+        if labels is not None:
+            loss_fct = BCEWithLogitsLoss(pos_weight=self.pos_weight)
+            labels = labels.float()
+            loss = loss_fct(
+                logits.view(-1, self.num_labels), labels.view(-1, self.num_labels)
+            )
+            outputs = (loss,) + outputs
+
+        return outputs  # (loss), logits, (hidden_states), (attentions)
+
+
 class RemBertForMultiLabelSequenceClassification(RemBertPreTrainedModel):
     """
     Bert model adapted for multi-label sequence classification
